@@ -3,7 +3,6 @@ import sys
 
 from pipenv import environments
 from pipenv.__version__ import __version__
-from pipenv._compat import fix_utf8
 from pipenv.cli.options import (
     CONTEXT_SETTINGS,
     PipenvGroup,
@@ -16,28 +15,30 @@ from pipenv.cli.options import (
     pypi_mirror_option,
     python_option,
     site_packages_option,
-    skip_lock_option,
     sync_options,
     system_option,
-    three_option,
     uninstall_options,
+    upgrade_options,
     verbose_option,
 )
+from pipenv.utils import console, err
 from pipenv.utils.environment import load_dot_env
 from pipenv.utils.processes import subprocess_run
 from pipenv.vendor.click import (
     Choice,
     argument,
-    echo,
     edit,
     group,
     option,
     pass_context,
-    secho,
-    style,
     version_option,
 )
 
+with console.capture() as capture:
+    console.print("[bold]pipenv[/bold]", end="")
+
+prog_name = capture.get()
+
 subcommand_context = CONTEXT_SETTINGS.copy()
 subcommand_context.update({"ignore_unknown_options": True, "allow_extra_args": True})
 subcommand_context_no_interspersion = subcommand_context.copy()
@@ -62,7 +63,7 @@ subcommand_context_no_interspersion["allow_interspersed_args"] = False
     help="Output diagnostic information for use in GitHub issues.",
 )
 @general_options
-@version_option(prog_name=style("pipenv", bold=True), version=__version__)
+@version_option(prog_name=prog_name, version=__version__)
 @pass_state
 @pass_context
 def cli(
@@ -80,26 +81,19 @@ def cli(
     site_packages=None,
     **kwargs,
 ):
-    from pipenv.patched.pip._vendor import rich
     from pipenv.utils.shell import system_which
 
     load_dot_env(state.project, quiet=state.quiet)
 
-    from ..core import (
-        cleanup_virtualenv,
-        do_clear,
-        do_py,
-        do_where,
-        ensure_project,
-        format_help,
-        warn_in_virtualenv,
-    )
+    from pipenv.routines.clear import do_clear
+    from pipenv.utils.display import format_help
+    from pipenv.utils.project import ensure_project
+    from pipenv.utils.virtualenv import cleanup_virtualenv, do_where, warn_in_virtualenv
 
     if "PIPENV_COLORBLIND" in os.environ:
-        echo(
+        err.print(
             "PIPENV_COLORBLIND is deprecated, use NO_COLOR"
             " per https://no-color.org/ instead",
-            err=True,
         )
 
     if man:
@@ -108,25 +102,20 @@ def cli(
             os.execle(system_which("man"), "man", path, os.environ)
             return 0
         else:
-            secho(
-                "man does not appear to be available on your system.",
-                fg="yellow",
-                bold=True,
-                err=True,
+            err.print(
+                "man does not appear to be available on your system.", style="bold yellow"
             )
             return 1
     if envs:
-        echo("The following environment variables can be set, to do various things:\n")
+        console.print(
+            "The following environment variables can be set, to do various things:\n"
+        )
         for key in state.project.__dict__:
             if key.startswith("PIPENV"):
-                echo(f"  - {style(key, bold=True)}")
-        echo(
-            "\nYou can learn more at:\n   {}".format(
-                style(
-                    "https://pipenv.pypa.io/en/latest/advanced/#configuration-with-environment-variables",
-                    fg="green",
-                )
-            )
+                console.print(f"  - {key}", style="bold")
+        console.print(
+            "\nYou can learn more at:\n   "
+            "[green]https://pipenv.pypa.io/en/latest/advanced/#configuration-with-environment-variables[/green]",
         )
         return 0
     warn_in_virtualenv(state.project)
@@ -152,64 +141,45 @@ def cli(
         elif venv:
             # There is no virtualenv yet.
             if not state.project.virtualenv_exists:
-                echo(
-                    "{}({}){}".format(
-                        style(
-                            "No virtualenv has been created for this project", fg="red"
-                        ),
-                        style(state.project.project_directory, bold=True),
-                        style(" yet!", fg="red"),
-                    ),
-                    err=True,
+                err.print(
+                    "[red]No virtualenv has been created for this project[/red]"
+                    f"[bold]{state.project.project_directory}[/bold]"
+                    " [red]yet![/red]"
                 )
                 ctx.abort()
             else:
-                echo(state.project.virtualenv_location)
+                print(state.project.virtualenv_location)
                 return 0
         # --rm was passed...
         elif rm:
             # Abort if --system (or running in a virtualenv).
             if state.project.s.PIPENV_USE_SYSTEM or environments.is_in_virtualenv():
-                echo(
-                    style(
-                        "You are attempting to remove a virtualenv that "
-                        "Pipenv did not create. Aborting.",
-                        fg="red",
-                    )
+                console.print(
+                    "You are attempting to remove a virtualenv that "
+                    "Pipenv did not create. Aborting.",
+                    style="red",
                 )
                 ctx.abort()
             if state.project.virtualenv_exists:
                 loc = state.project.virtualenv_location
-                echo(
-                    style(
-                        "{} ({})...".format(
-                            style("Removing virtualenv", bold=True),
-                            style(loc, fg="green"),
-                        )
-                    )
+                console.print(
+                    f"[bold]Removing virtualenv[/bold] ([green]{loc}[green])..."
                 )
 
-                console = rich.console.Console()
-                # TODO: add state.project.s to spinner status
                 with console.status("Running..."):
                     # Remove the virtualenv.
                     cleanup_virtualenv(state.project, bare=True)
                 return 0
             else:
-                echo(
-                    style(
-                        "No virtualenv has been created for this project yet!",
-                        fg="red",
-                        bold=True,
-                    ),
-                    err=True,
+                err.print(
+                    "No virtualenv has been created for this project yet!",
+                    style="red bold",
                 )
                 ctx.abort()
-    # --python or --three was passed...
-    if (state.python or state.three is not None) or state.site_packages:
+    # --python was passed...
+    if (state.python) or state.site_packages:
         ensure_project(
             state.project,
-            three=state.three,
             python=state.python,
             warn=True,
             site_packages=state.site_packages,
@@ -219,7 +189,7 @@ def cli(
     # Check this again before exiting for empty ``pipenv`` command.
     elif ctx.invoked_subcommand is None:
         # Display help to user, if no commands were passed.
-        echo(format_help(ctx.get_help()))
+        print(format_help(ctx.get_help()))
 
 
 @cli.command(
@@ -229,33 +199,65 @@ def cli(
 @system_option
 @deploy_option
 @site_packages_option
-@skip_lock_option
 @install_options
 @pass_state
 def install(state, **kwargs):
-    """Installs provided packages and adds them to Pipfile, or (if no packages are given), installs all packages from Pipfile."""
-    from ..core import do_install
+    """Installs provided packages and adds them to Pipfile,
+    or (if no packages are given), installs all packages from Pipfile."""
+    from pipenv.routines.install import do_install
 
     do_install(
         state.project,
         dev=state.installstate.dev,
-        three=state.three,
         python=state.python,
         pypi_mirror=state.pypi_mirror,
         system=state.system,
         ignore_pipfile=state.installstate.ignore_pipfile,
-        skip_lock=state.installstate.skip_lock,
         requirementstxt=state.installstate.requirementstxt,
         pre=state.installstate.pre,
         deploy=state.installstate.deploy,
-        keep_outdated=state.installstate.keep_outdated,
-        selective_upgrade=state.installstate.selective_upgrade,
-        index_url=state.index,
+        index=state.index,
         packages=state.installstate.packages,
         editable_packages=state.installstate.editables,
         site_packages=state.site_packages,
         extra_pip_args=state.installstate.extra_pip_args,
         categories=state.installstate.categories,
+        skip_lock=state.installstate.skip_lock,
+    )
+
+
+@cli.command(
+    short_help="Resolves provided packages and adds them to Pipfile, or (if no packages are given), merges results to Pipfile.lock",
+    context_settings=subcommand_context,
+)
+@system_option
+@site_packages_option
+@install_options
+@upgrade_options
+@pass_state
+def upgrade(state, **kwargs):
+    from pipenv.routines.update import upgrade
+    from pipenv.utils.project import ensure_project
+
+    ensure_project(
+        state.project,
+        python=state.python,
+        pypi_mirror=state.pypi_mirror,
+        warn=(not state.quiet),
+        site_packages=state.site_packages,
+        clear=state.clear,
+    )
+
+    upgrade(
+        state.project,
+        pre=state.installstate.pre,
+        packages=state.installstate.packages,
+        editable_packages=state.installstate.editables,
+        categories=state.installstate.categories,
+        index_url=state.index,
+        dev=state.installstate.dev,
+        system=state.system,
+        lock_only=state.installstate.lock_only,
     )
 
 
@@ -280,19 +282,17 @@ def install(state, **kwargs):
 @pass_context
 def uninstall(ctx, state, all_dev=False, all=False, **kwargs):
     """Uninstalls a provided package and removes it from Pipfile."""
-    from ..core import do_uninstall
+    from pipenv.routines.uninstall import do_uninstall
 
     retcode = do_uninstall(
         state.project,
         packages=state.installstate.packages,
         editable_packages=state.installstate.editables,
-        three=state.three,
         python=state.python,
         system=state.system,
-        lock=not state.installstate.skip_lock,
+        lock=True,
         all_dev=all_dev,
         all=all,
-        keep_outdated=state.installstate.keep_outdated,
         pypi_mirror=state.pypi_mirror,
         categories=state.installstate.categories,
         ctx=ctx,
@@ -306,7 +306,7 @@ LOCK_HEADER = """\
 # These requirements were autogenerated by pipenv
 # To regenerate from the project's Pipfile, run:
 #
-#    pipenv lock {options}
+#    pipenv requirements {options}
 #
 """
 
@@ -323,14 +323,14 @@ LOCK_DEV_NOTE = """\
 @pass_context
 def lock(ctx, state, **kwargs):
     """Generates Pipfile.lock."""
-    from ..core import do_lock, ensure_project
+    from pipenv.routines.lock import do_lock
+    from pipenv.utils.project import ensure_project
 
     # Ensure that virtualenv is available.
     # Note that we don't pass clear on to ensure_project as it is also
     # handled in do_lock
     ensure_project(
         state.project,
-        three=state.three,
         python=state.python,
         pypi_mirror=state.pypi_mirror,
         warn=(not state.quiet),
@@ -339,10 +339,8 @@ def lock(ctx, state, **kwargs):
     pre = state.installstate.pre
     do_lock(
         state.project,
-        ctx=ctx,
         clear=state.clear,
         pre=pre,
-        keep_outdated=state.installstate.keep_outdated,
         pypi_mirror=state.pypi_mirror,
         write=not state.quiet,
         categories=state.installstate.categories,
@@ -368,7 +366,6 @@ def lock(ctx, state, **kwargs):
 )
 @argument("shell_args", nargs=-1)
 @pypi_mirror_option
-@three_option
 @python_option
 @pass_state
 def shell(
@@ -378,20 +375,17 @@ def shell(
     anyway=False,
 ):
     """Spawns a shell within the virtualenv."""
-    from ..core import do_shell
+    from pipenv.routines.shell import do_shell
 
     # Prevent user from activating nested environments.
     if "PIPENV_ACTIVE" in os.environ:
         # If PIPENV_ACTIVE is set, VIRTUAL_ENV should always be set too.
         venv_name = os.environ.get("VIRTUAL_ENV", "UNKNOWN_VIRTUAL_ENVIRONMENT")
         if not anyway:
-            echo(
-                "{} {} {}\nNo action taken to avoid nested environments.".format(
-                    style("Shell for"),
-                    style(venv_name, fg="green", bold=True),
-                    style("already activated.", bold=True),
-                ),
-                err=True,
+            err.print(
+                f"Shell for [green bold]{venv_name}[/green bold] "
+                "[bold]already activated[/bold].\n"
+                "New shell not activated to avoid nested environments."
             )
             sys.exit(1)
     # Load .env file.
@@ -401,7 +395,6 @@ def shell(
         fancy = True
     do_shell(
         state.project,
-        three=state.three,
         python=state.python,
         fancy=fancy,
         shell_args=shell_args,
@@ -419,13 +412,12 @@ def shell(
 @pass_state
 def run(state, command, args):
     """Spawns a command installed into the virtualenv."""
-    from ..core import do_run
+    from pipenv.routines.shell import do_run
 
     do_run(
         state.project,
         command=command,
         args=args,
-        three=state.three,
         python=state.python,
         pypi_mirror=state.pypi_mirror,
     )
@@ -486,13 +478,23 @@ def run(state, command, args):
     help="Path to where output file will be placed, if the path is a directory, "
     "Safety will use safety-report.json as filename. Default: empty",
 )
+@option(
+    "--use-installed",
+    is_flag=True,
+    help="Whether to use the lockfile as input to check (instead of result from pip list).",
+)
+@option(
+    "--categories",
+    is_flag=False,
+    default="",
+    help="Use the specified categories from the lockfile as input to check.",
+)
 @common_options
 @system_option
 @pass_state
 def check(
     state,
     db=None,
-    style=False,
     ignore=None,
     output="screen",
     key=None,
@@ -502,14 +504,15 @@ def check(
     save_json="",
     audit_and_monitor=True,
     project=None,
+    use_installed=False,
+    categories="",
     **kwargs,
 ):
     """Checks for PyUp Safety security vulnerabilities and against PEP 508 markers provided in Pipfile."""
-    from ..core import do_check
+    from pipenv.routines.check import do_check
 
     do_check(
         state.project,
-        three=state.three,
         python=state.python,
         system=state.system,
         db=db,
@@ -523,6 +526,8 @@ def check(
         audit_and_monitor=audit_and_monitor,
         safety_project=project,
         pypi_mirror=state.pypi_mirror,
+        use_installed=use_installed,
+        categories=categories,
     )
 
 
@@ -531,75 +536,32 @@ def check(
 @option("--outdated", is_flag=True, default=False, help="List out-of-date dependencies.")
 @option("--dry-run", is_flag=True, default=None, help="List out-of-date dependencies.")
 @install_options
+@upgrade_options
 @pass_state
 @pass_context
 def update(ctx, state, bare=False, dry_run=None, outdated=False, **kwargs):
-    """Runs lock, then sync."""
-    from ..core import do_lock, do_outdated, do_sync, ensure_project
+    """Runs lock when no packages are specified, or upgrade, and then sync."""
+    from pipenv.routines.update import do_update
 
-    ensure_project(
+    do_update(
         state.project,
-        three=state.three,
         python=state.python,
-        pypi_mirror=state.pypi_mirror,
-        warn=(not state.quiet),
         site_packages=state.site_packages,
         clear=state.clear,
-    )
-    if not outdated:
-        outdated = bool(dry_run)
-    if outdated:
-        do_outdated(
-            state.project,
-            clear=state.clear,
-            pre=state.installstate.pre,
-            pypi_mirror=state.pypi_mirror,
-        )
-    packages = [p for p in state.installstate.packages if p]
-    editable = [p for p in state.installstate.editables if p]
-    if not packages:
-        echo(
-            "{} {} {} {}{}".format(
-                style("Running", bold=True),
-                style("$ pipenv lock", fg="yellow", bold=True),
-                style("then", bold=True),
-                style("$ pipenv sync", fg="yellow", bold=True),
-                style(".", bold=True),
-            )
-        )
-    else:
-        for package in packages + editable:
-            if package not in state.project.all_packages:
-                echo(
-                    "{}: {} was not found in your Pipfile! Aborting."
-                    "".format(
-                        style("Warning", fg="red", bold=True),
-                        style(package, fg="green", bold=True),
-                    ),
-                    err=True,
-                )
-                ctx.abort()
-    do_lock(
-        state.project,
-        ctx=ctx,
-        clear=state.clear,
         pre=state.installstate.pre,
-        keep_outdated=state.installstate.keep_outdated,
         pypi_mirror=state.pypi_mirror,
-        write=not state.quiet,
-    )
-    do_sync(
-        state.project,
+        system=False,
+        packages=state.installstate.packages,
+        editable_packages=state.installstate.editables,
         dev=state.installstate.dev,
-        three=state.three,
-        python=state.python,
         bare=bare,
-        dont_upgrade=not state.installstate.keep_outdated,
-        user=False,
-        clear=state.clear,
-        unused=False,
-        pypi_mirror=state.pypi_mirror,
         extra_pip_args=state.installstate.extra_pip_args,
+        categories=state.installstate.categories,
+        index_url=state.index,
+        quiet=state.quiet,
+        dry_run=dry_run,
+        outdated=outdated,
+        lock_only=state.installstate.lock_only,
     )
 
 
@@ -614,7 +576,7 @@ def update(ctx, state, bare=False, dry_run=None, outdated=False, **kwargs):
 @pass_state
 def graph(state, bare=False, json=False, json_tree=False, reverse=False):
     """Displays currently-installed dependency graph information."""
-    from ..core import do_graph
+    from pipenv.routines.graph import do_graph
 
     do_graph(state.project, bare=bare, json=json, json_tree=json_tree, reverse=reverse)
 
@@ -635,12 +597,12 @@ def run_open(state, module, *args, **kwargs):
 
         EDITOR=atom pipenv open requests
     """
-    from ..core import ensure_project, inline_activate_virtual_environment
+    from pipenv.utils.project import ensure_project
+    from pipenv.utils.virtualenv import inline_activate_virtual_environment
 
     # Ensure that virtualenv is available.
     ensure_project(
         state.project,
-        three=state.three,
         python=state.python,
         validate=False,
         pypi_mirror=state.pypi_mirror,
@@ -649,17 +611,17 @@ def run_open(state, module, *args, **kwargs):
         [
             state.project._which("python"),
             "-c",
-            "import {0}; print({0}.__file__)".format(module),
+            f"import {module}; print({module}.__file__)",
         ]
     )
     if c.returncode:
-        echo(style("Module not found!", fg="red"))
+        console.print("Module not found!", style="red")
         sys.exit(1)
     if "__init__.py" in c.stdout:
         p = os.path.dirname(c.stdout.strip().rstrip("cdo"))
     else:
         p = c.stdout.strip().rstrip("cdo")
-    echo(style(f"Opening {p!r} in your EDITOR.", bold=True))
+    console.print(f"Opening {p!r} in your EDITOR.", style="bold")
     inline_activate_virtual_environment(state.project)
     edit(filename=p)
     return 0
@@ -672,19 +634,18 @@ def run_open(state, module, *args, **kwargs):
 @system_option
 @option("--bare", is_flag=True, default=False, help="Minimal output.")
 @sync_options
+@site_packages_option
 @pass_state
 @pass_context
 def sync(ctx, state, bare=False, user=False, unused=False, **kwargs):
     """Installs all packages specified in Pipfile.lock."""
-    from ..core import do_sync
+    from pipenv.routines.install import do_sync
 
     retcode = do_sync(
         state.project,
         dev=state.installstate.dev,
-        three=state.three,
         python=state.python,
         bare=bare,
-        dont_upgrade=(not state.installstate.keep_outdated),
         user=user,
         clear=state.clear,
         unused=unused,
@@ -692,6 +653,7 @@ def sync(ctx, state, bare=False, user=False, unused=False, **kwargs):
         system=state.system,
         extra_pip_args=state.installstate.extra_pip_args,
         categories=state.installstate.categories,
+        site_packages=state.site_packages,
     )
     if retcode:
         ctx.abort()
@@ -704,16 +666,14 @@ def sync(ctx, state, bare=False, user=False, unused=False, **kwargs):
 @option("--bare", is_flag=True, default=False, help="Minimal output.")
 @option("--dry-run", is_flag=True, default=False, help="Just output unneeded packages.")
 @verbose_option
-@three_option
 @python_option
 @pass_state
 def clean(state, dry_run=False, bare=False, user=False):
     """Uninstalls all packages not specified in Pipfile.lock."""
-    from ..core import do_clean
+    from pipenv.routines.clean import do_clean
 
     do_clean(
         state.project,
-        three=state.three,
         python=state.python,
         dry_run=dry_run,
         system=state.system,
@@ -729,18 +689,17 @@ def clean(state, dry_run=False, bare=False, user=False):
 def scripts(state):
     """Lists scripts in current environment config."""
     if not state.project.pipfile_exists:
-        echo("No Pipfile present at project home.", err=True)
+        err.print("No Pipfile present at project home.")
         sys.exit(1)
     scripts = state.project.parsed_pipfile.get("scripts", {})
     first_column_width = max(len(word) for word in ["Command"] + list(scripts))
     second_column_width = max(len(word) for word in ["Script"] + list(scripts.values()))
-    lines = ["{0:<{width}}  Script".format("Command", width=first_column_width)]
-    lines.append("{}  {}".format("-" * first_column_width, "-" * second_column_width))
+    lines = [f"{command:<{first_column_width}}  Script" for command in ["Command"]]
+    lines.append(f"{'-' * first_column_width}  {'-' * second_column_width}")
     lines.extend(
-        "{0:<{width}}  {1}".format(name, script, width=first_column_width)
-        for name, script in scripts.items()
+        f"{name:<{first_column_width}}  {script}" for name, script in scripts.items()
     )
-    echo("\n".join(fix_utf8(line) for line in lines))
+    console.print("\n".join(line for line in lines))
 
 
 @cli.command(
@@ -751,17 +710,14 @@ def scripts(state):
 def verify(state):
     """Verify the hash in Pipfile.lock is up-to-date."""
     if not state.project.pipfile_exists:
-        echo("No Pipfile present at project home.", err=True)
+        err.print("No Pipfile present at project home.")
         sys.exit(1)
     if state.project.get_lockfile_hash() != state.project.calculate_pipfile_hash():
-        echo(
-            "Pipfile.lock is out-of-date. Run {} to update.".format(
-                style("$ pipenv lock", fg="yellow", bold=True)
-            ),
-            err=True,
+        err.print(
+            "Pipfile.lock is out-of-date. Run [yellow bold]$ pipenv lock[/yellow bold] to update."
         )
         sys.exit(1)
-    echo(style("Pipfile.lock is up-to-date.", fg="green"))
+    console.print("Pipfile.lock is up-to-date.", style="green")
     sys.exit(0)
 
 
@@ -785,41 +741,32 @@ def verify(state):
 def requirements(
     state, dev=False, dev_only=False, hash=False, exclude_markers=False, categories=""
 ):
+    from pipenv.routines.requirements import generate_requirements
 
-    from pipenv.utils.dependencies import convert_deps_to_pip
-
-    lockfile = state.project.load_lockfile(expand_env_vars=False)
-
-    for i, package_index in enumerate(lockfile["_meta"]["sources"]):
-        prefix = "-i" if i == 0 else "--extra-index-url"
-        echo(" ".join([prefix, package_index["url"]]))
-
-    deps = {}
-    categories_list = categories.split(",") if categories else []
-
-    if categories_list:
-        for category in categories_list:
-            category = category.strip()
-            deps.update(lockfile.get(category, {}))
-    else:
-        if dev or dev_only:
-            deps.update(lockfile["develop"])
-        if not dev_only:
-            deps.update(lockfile["default"])
-
-    pip_deps = convert_deps_to_pip(
-        deps,
-        project=None,
-        include_index=False,
+    generate_requirements(
+        project=state.project,
+        dev=dev,
+        dev_only=dev_only,
         include_hashes=hash,
         include_markers=not exclude_markers,
+        categories=categories,
     )
 
-    for d in pip_deps:
-        echo(d)
-
-    sys.exit(0)
-
 
 if __name__ == "__main__":
     cli()
+
+
+def do_py(project, ctx=None, system=False):
+    if not project.virtualenv_exists:
+        err.print(
+            "[red]No virtualenv has been created for this project[/red] "
+            f"[yellow bold]{project.project_directory}[/yellow bold] "
+            "[red] yet![/red]"
+        )
+        ctx.abort()
+
+    try:
+        console.print(project._which("python", allow_global=system))
+    except AttributeError:
+        console.print("No project found!", style="red")
